# ifndef FORMULA_STAMP_H
# define FORMULA_STAMP_H

# include <vector>

# include "formula.h"


/**
 *
 * This class represents holds a reference for another formula
 */
class FormulaStamp: public Formula {
	public:
		/*
		 * @param stamp: the ID of the formula this FormulaStamp points to.
		 *              This must be a valid ID from the global formula table
		 *              (the variable 'table').
		 */
		FormulaStamp (int stamp);
		~FormulaStamp ();

		/*
		 * Returns the number of elements of the formula pointed by this instance
		 * of FormulaStamp.
		 */
		int getNumberOfElements();

		/*
		 * @param index: the index of the element to be retrieved.
		 * Returns the element at index of the formula pointed by this instance
		 * of FormulaStamp, if possible. Returns NULL otherwise.
		 */
		Formula* getElement (int index);

		/*
		 * Returns the IDValue of the formula pointed by this instance
		 * of FormulaStamp, if possible. Returns -1 otherwise.
		 */
		int getIDValue ();

		/*
		 * @param f: the formula to be compared.
		 * Returns true if f is a FormulaStamp and points to the same formula
		 * that this instance points. Returns false otherwise.
		 */
		bool equals (Formula *f);

		/*
		 * @param v: the assignment to be applied.
		 * Returns the value of the formula pointed by this instance
		 * of FormulaStamp, when v is applied. Returns false if no formula is
		 * pointed
		 */
		bool getValue (Assignment *v);

		/*
		 * prints the formula pointed by this instance of FormulaStamp in the
		 * standard output.
		 */
		void print();

		/*
		 * Returns the ID of the formula pointed by this instance of
		 * FormulaStamp. This value is associated with the global formula table
		 */
		int getStamp();

	private:
		int stamp; };



# endif // FROMULA_STAMP_H
