# include <ctime>
# include "utilities.h"

# include "../parser/parser.h"
# include "../formula/formula.h"
# include "../formula/formulaStamp.h"
# include "../map/IDmap.h"
# include "../map/assignment.h"
# include "formulaCalc.h"


static void testSequent (Formula *sequent, Map *m);
static void satisfyFormula (Formula *formula, Map *m);
static void printSolution (Map *m, Assignment *v);


/*
 * @param formula: the formula to be calculated.
 * @param m: the map that associates variables IDs to strings (the variables
 *			original names).
 * Calculates the requested formula and prints the result on the standard output
 * May generate extra profile analysis in a specified file output.
 */
void calcFormula (Formula *formula, Map *m) {
	if (formula == NULL) {
		if (getSilence() < 3) {
			std::cout << "could not read the formula" << std::endl << std::endl; }
		return; }

	if (getSilence() < 1) {
		formula->print();
		std::cout << std::endl; }

	if ((dynamic_cast<FormulaStamp *>(formula) != NULL &&
		dynamic_cast<Sequent *>(table->getFormula
		((dynamic_cast<FormulaStamp *>(formula))->getStamp())))
		|| dynamic_cast<Sequent *>(formula) != NULL) testSequent (formula, m);
	else satisfyFormula (formula, m);

	formula->erase();
	return; }


static void testSequent (Formula *sequent, Map *m) {
	Assignment *v = new Assignment(m->numberOfElements());

	for ( ; !(v->isAllValuesTrue()); v->nextValueSet()) {
		if (sequent->getValue(v) == false) {
			if (getSilence() < 3) {
				std::cout << "sequent is not a tautology" << std::endl; }
			delete v;
			return; }}

	delete v;
	if (getSilence() < 3) {
	    std::cout << "sequent is a tautology" << std::endl; }}


static void satisfyFormula (Formula *formula, Map *m) {
	Assignment *v = new Assignment(m->numberOfElements());

   	for ( ; !(v->isAllValuesTrue()); v->nextValueSet()) {
		if (formula->getValue(v) == true) {
			printSolution(m, v);
			formula->erase();
			delete v;
			return; }}

	if (getSilence() < 3) {
		if (formula->getValue(v) == true) printSolution(m, v);
		else std::cout << "solution not found" << std::endl; }}


static void printSolution (Map *m, Assignment *v) {
	std::cout << "solution found:" << std::endl;
	std::string value;
	for (int i = 0; i < m->numberOfElements(); i++) {
		if (v->getValue(i)) value = "true";
		else value = "false";
		std::cout << "\t" << *(m->getElement(i)) + ": " + value << std::endl; }
	std::cout << std::endl; }


